/**
 * Created by lixiaodong on 16/7/12.
 */
var async = require('async');

var redisUtil = require('./redis/redisUtil.js');
var redisKeyManager = require('./redis/redisKeyManager.js');

var rankingModel = function (opt) {
    this.redisKeyManager = new redisKeyManager(opt);
    this.redisManager = new redisUtil(opt);
    this.rankingSetKey = this.redisKeyManager.getRedisKey(opt.redis.RANKING_SET_KEY);
    this.init(opt);
}

var prototype = rankingModel.prototype;

//init pvp robot
rankingModel.prototype.init = function (opt) {
    var self = this;
    //init pvp robot data to ranking
    if(opt.robots){//[{self : 'robot_1001', ranking: 1},{robotId : 'robot_1002', ranking: 2},,,,]
        var sets = [self.rankingSetKey];

        for(var i = 0 ; i < opt.robots.length; i++){
            sets.push(opt.robots[i].ranking);
            sets.push(opt.robots[i].robotId);
        }
        console.log('init pvp ranking');
        self.redisManager.redisZAddSets(sets, function (err) {});
    }
}

/**
 * return err ranking
 * @param id
 * @param next
 */
rankingModel.prototype.addToRanking = function (id,next) {
    var self = this;

    var sets = [self.rankingSetKey];
    var max = 10000;

    async.waterfall([
        function (cb) {
            //if id in ranking return;
            self.getRankingById( id, function (err,data) {
                if(!err && !!data){
                    next(null,data);
                    return;
                }
                cb(err);
            });
        },
        function (cb) {
            self.redisManager.redisZCard(self.rankingSetKey, function (err,data) {
                if(!err && !!data){
                    max = data + 1;
                }
                cb(err);
            });
        },
        function (cb) {
            sets.push(max);
            sets.push(id);
            self.redisManager.redisZAddSets(sets, function (err) {
                cb(err);
            });
        }
    ], function (err) {
        next(err,max);
    });
}

//根据ID 获取排名
rankingModel.prototype.getRankingById = function (id, next) {
    var self = this;
    self.redisManager.redisZSCORE(self.rankingSetKey, id , function (err,data) {
        next(err,data);
    });
}

//交换排行 返回idA最新的排名
rankingModel.prototype.changeRankingOfTwoPlayer = function (idA,idB,next) {
    var self = this;

    var rankingObj = {};

    async.waterfall([
        function (cb) {
            self.getRankingById(idA, function (err,ranking) {
                if(!err && !!ranking){
                    rankingObj.oldPlayerARanking = ranking;
                }
                cb(err);
            });
        },
        function (cb) {
            self.getRankingById(idB, function (err,ranking) {
                if(!err && !!ranking){
                    rankingObj.oldPlayerBRanking = ranking;
                }
                cb(err);
            });
        },
        function (cb) {
            //if ranking a is bigger than ranking b , no change ranking
            if(rankingObj.oldPlayerARanking < rankingObj.oldPlayerBRanking){
                next(null,rankingObj.oldPlayerARanking);
                return;
            }
            var sets = [self.rankingSetKey];

            sets.push(rankingObj.oldPlayerBRanking);
            sets.push(idA);

            sets.push(rankingObj.oldPlayerARanking);
            sets.push(idB);
            self.redisManager.redisZAddSets(sets, function (err) {
                cb(err);
            });
        }
    ], function (err) {
        next(err,rankingObj.oldPlayerBRanking);
    });
}

/**
 * 获取排行榜前max名
 * @param max
 * @param next
 */
rankingModel.prototype.getTopRanking = function(max, next){
    var self = this;
    var listOfRankingPlayerId = [];
    self.redisManager.redisZRANGEBYSCORE(self.rankingSetKey, 1, max, function (err,data) {
        if(!err && !!data){
            listOfRankingPlayerId = data;
        }
        next(err,listOfRankingPlayerId);
    });
}


module.exports = rankingModel;