/**
 * Created by lixiaodong on 16/5/17.
 */
var ioredis = require('ioredis');

var RedisUtil = function (opt) {
    this.client = null;
    this.init(opt);
};

var prototype = RedisUtil.prototype;

prototype.init = function (opt) {
    console.log('init redis util');
    if(opt.redis.cluster_mode){//create redis cluster
        this.redisClient = new ioredis.Cluster(opt.redis.cluster);
        console.log('use redis with cluster');
    } else {
        this.redisClient = new ioredis(opt.redis.single);
        console.log('use redis with single point');
    }

    this.redisClient.on('connect',function(){
        console.log("REDIS CONNECTED");
    });
    this.redisClient.on('ready',function(){
        console.log("REDIS READY");
    });
    this.redisClient.on('error',function(err){
        console.log("REDIS CONNECTION error "+ err);
        console.log('node error', err.lastNodeError);
    });
    this.redisClient.on('close',function(){
        console.log("REDIS CONNECTION CLOSE");
    });
    this.redisClient.on('reconnecting',function(){
        console.log("REDIS RECONNECTING");
    });
    this.redisClient.on('end',function(){
        console.log("REDIS CONNECTION END");
    });
};

prototype.setRedisData = function (key,value,callback){
    this.redisClient.set(key,JSON.stringify(value), function (err) {
        callback(err);
    });
}

prototype.setRedisDataOfDays = function (key,value,days,callback){
    this.redisClient.set(key,JSON.stringify(value), function () {
        this.redisClient.expire(key, days * 24 * 60 * 60 * 1000, callback);
    });
}

prototype.setRedisDataOfSeconds = function (key,value,seconds,callback){
    this.redisClient.setex(key,seconds,JSON.stringify(value), function(err) {
        callback(err);
    });
};

prototype.getRedisData = function (key,callback){
    var redisData;

    this.redisClient.get(key,function(err,data){
        if(!err && !!data){
            redisData = JSON.parse(data);
        }
        callback(err,redisData);
    });
}


//获取redis中所有玩家的uid
prototype.getRedisKeys = function (key,cb){
    this.redisClient.keys(key, function (err,keys) {
        cb(err,keys);
    });
}
prototype.getRedisClient = function() {
    return this.redisClient;
}

//zadd range
prototype.redisZAdd = function(key,score,uid,callback){
    this.redisClient.zadd(key,score,uid, callback);
}

prototype.redisZAddSets = function(sets,callback){
    this.redisClient.zadd(sets, function (err) {
        callback(err);
    });
}

//get zrange
prototype.redisZRANGEBYSCORE = function(key,min,max,cb){
    this.redisClient.zrangebyscore(key,min,max, function (err,data) {
        cb(err,data);
    });
}

prototype.redisZRANGEBYSCOREALL = function(key,cb){
    this.redisClient.zrangebyscore(key,'-inf','+inf', function (err,data) {
        cb(err,data);
    });
}


prototype.redisZSCORE = function(key,uid,cb){
    this.redisClient.zscore(key,uid, function (err,data) {
        cb(err,data);
    });
}

prototype.redisZCard = function(key,callback){
    this.redisClient.zcard(key, function (err,data) {
        callback(err,data);
    });
}

//获取redis中所有玩家的uid
prototype.getRedisKeys = function(key,cb){
    this.redisClient.keys(key, function (err,keys) {
        cb(err,keys);
    });
}

prototype.redisMget = function(keys,cb){
    this.redisClient.mget(keys, function (err,data) {
        if(!err && !!data){
            data = JSON.parse(data);
        }
        cb(err,data);
    });
}

prototype.redisMset = function(sets,callback){
    this.redisClient.mset(sets,callback);
}

prototype.redisHset = function(key,field,value,callback){
    this.redisClient.hset(key,field,JSON.stringify(value),callback);
}

prototype.redisHMGet = function(key,fields,callback){
    this.redisClient.hmget(key,fields, function (err,data) {
        var array = [];
        if(!err && !!data && data.length){
            for(var i = 0 ; i < data.length; i++){
                array.push(JSON.parse(data[i]));
            }
        }
        callback(err,array);
    });
}

prototype.redisHKeys = function(key,callback){
    this.redisClient.hkeys(key,callback);
}

prototype.redisHMset = function(sets,callback){
    this.redisClient.hmset(sets,callback);
}

prototype.redisHGet = function(key,field,callback){

    this.redisClient.hget(key,field, function (err,data) {
        if(!err && !!data){
            data = JSON.parse(data);
        }
        callback(err,data);
    });
}

module.exports = RedisUtil;
